﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

namespace WifiLocator
{
    public class Client
    {
        int max_tolerance_signal = 15;

        public Client(int id, string name, string color, int x, int y) {
            this.Id = id;
            this.Name = name;
            this.Color = color;
            this.Position = new Point(x, y);
            newData = false;
        }

        public void display(Canvas canvas, double meritko, double posunX, double posunY) {
            Ellipse ellipse = new Ellipse();
            BrushConverter bc = new BrushConverter();
            ellipse.Stroke = (Brush)bc.ConvertFrom(this.Color);
            ellipse.Fill = (Brush)bc.ConvertFrom(this.Color);
            ellipse.Width = 30*meritko;
            ellipse.Height = 30*meritko;
            Canvas.SetLeft(ellipse, (this.Position.X * meritko) + posunX);
            Canvas.SetTop(ellipse, (this.Position.Y * meritko) + posunY);
            canvas.Children.Add(ellipse);
        }

        public void parseMessage(string msg) {
            this.time = DateTime.Now;
            this.newData = true;
            this.signals = new List<Signal>();
            string[] hotspots = Regex.Split(msg,";");
            foreach (string hotspot in hotspots) {
                 string pattern = @"^(?<ssid>[^\(]+)\((?<msg>[^\)]+)\)$";
                 Regex r = new Regex(pattern, RegexOptions.IgnoreCase);
                 Match m = r.Match(hotspot);
                 if (m.Success)
                 {
                    string[] data = Regex.Split(m.Groups["msg"].Value, ",");
                    List<string> dataList = new List<string>();
                    foreach (string x in data) {
                        dataList.Add(x);
                    }
                    if(dataList.Count == 4)
                    {
                        string ssid = m.Groups["ssid"].Value;
                        int strenght = Math.Abs(int.Parse(dataList[1]));
                        string mac = dataList[2];
                        Console.WriteLine("H:"+mac+" - "+strenght.ToString()+" - "+ssid);
                        Signal signal = new Signal(mac, strenght, ssid);
                        signals.Add(signal);
                    }
                }
            }
        }

        public void calculatePosition(List<SignalMap> signalMaps) {
            List<Point> points = new List<Point>();
            int tolerance_signal = 0;
            //vyhledani bodu v mapach
            do
            {
                foreach (SignalMap sm in signalMaps)
                {
                    Signal s = getSignal(sm.Mac); //signal dle mac
                    if (s != null && s.Strenght > 0)
                    {
                        //signal pro danou mapu...
                        List<Point> sp = sm.getPoints(s.Strenght, tolerance_signal);
                        //if (points == null)
                        //    points = sp;
                        //else
                        //{
                            // ponechava pouze spolecne body
                            /*
                            List<Point> newpoints = new List<Point>();
                            foreach (Point p1 in sp)
                            {
                                foreach (Point p2 in points)
                                {
                                    if (p1.X == p2.X && p1.Y == p2.Y)
                                    {
                                        newpoints.Add(p1);
                                    }
                                }
                            }
                            if (newpoints.Count > 0)
                                points = newpoints;
                            */

                            //bere vsechny mozne body
                            foreach (Point p1 in sp)
                            {
                                bool exists = false;
                                foreach (Point p2 in points)
                                {
                                    if (p1.X == p2.X && p1.Y == p2.Y)
                                    {
                                        exists = true;
                                        break;
                                    }
                                }
                                if (!exists)
                                    points.Add(p1);
                            }
                            /////////////////////////////////////////////
                        //}
                    }
                }
                tolerance_signal++;
            } while (points.Count == 0 && tolerance_signal <= max_tolerance_signal);

            if (points.Count == 0)
            {
                Console.WriteLine("Poloha nebyla určena!");
                Console.WriteLine("----------------------------------------------");
            }
            else if (points.Count == 1)
            {
                Console.WriteLine("Nová poloha");
                Console.WriteLine("----------------------------------------------");
                this.Position.X = points[0].X;
                this.Position.Y = points[0].Y;
                Console.WriteLine("Point({0},{1})", points[0].X, points[0].Y);
                Console.WriteLine("----------------------------------------------");
            }
            else
            {
                Console.WriteLine("Více bodů pro určení polohy");
                Console.WriteLine("----------------------------------------------");

                List<PointGroup> pointgroups = new List<PointGroup>();

                foreach (Point p in points) {
                    Console.Write("POINT ({0},{1})\t", p.X, p.Y);
                    bool newgroup = true;
                    foreach (PointGroup pg in pointgroups) {
                        if (pg.Add(p))
                        {
                            newgroup = false;
                            break;
                        }
                    }
                    if (newgroup) {
                        Console.WriteLine("NEW POINT GROUP");
                        PointGroup pg = new PointGroup();
                        pg.Add(p);
                        pointgroups.Add(pg);
                    }
                }

                if (pointgroups.Count == 1) {
                    Point np = pointgroups[0].getAverangePoint();
                    this.Position.X = np.X;
                    this.Position.Y = np.Y;
                } else if (pointgroups.Count > 1) {
                    Point pm = null;
                    int maxcount = 0;
                    foreach (PointGroup pg in pointgroups) {
                        if (pg.getCount() > maxcount) {
                            maxcount = pg.getCount();
                            pm = pg.getAverangePoint();
                        }
                    }
                    if (pm != null) {
                        this.Position.X = pm.X;
                        this.Position.Y = pm.Y;
                    }
                }

                Console.WriteLine("\nNew position ({0},{1}) from {2} points in {3} groups..",this.Position.X.ToString(),this.Position.Y.ToString(),points.Count.ToString(),pointgroups.Count.ToString());
                Console.WriteLine("----------------------------------------------");
            }
            Console.WriteLine("Tolerance: {0}",tolerance_signal);
        }

        private Signal getSignal(string mac) {
            if (signals == null)
                return null;
            foreach (Signal s in signals) {
                if (s.MacAddress == mac)
                    return s;
            }
            return null;
        }

        public int Id { get; }

        public string Name { get; }

        public string Color { get; }

        public Point Position { get; set; }

        public List<Signal> signals { get; set; }

        public DateTime time { get; private set; }

        public bool newData { get; set; }
    }
}
