﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Net;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace WifiLocator
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public IPAddress ip = IPAddress.Any;
        public Int32 port = 1100;

        public Map map;

        public bool showMousePosition = false;
        public System.Windows.Point MousePosition = new System.Windows.Point(0,0);

        private Server server;
        private Thread serverThread;

        public MainWindow()
        {
            InitializeComponent();
            updateTime();
            Closing += OnWindowClosing;

            DispatcherTimer timer = new DispatcherTimer(new TimeSpan(0, 0, 1), DispatcherPriority.Normal, delegate
            {
                this.updateTime();
            }, this.Dispatcher);

            DispatcherTimer printer = new DispatcherTimer(new TimeSpan(0,0,0,0,100), DispatcherPriority.Normal, delegate
            {
                this.display();
            }, this.Dispatcher);

            IPHostEntry ipHostInfo = Dns.GetHostEntry(Dns.GetHostName());
            foreach (IPAddress ip in ipHostInfo.AddressList)
            {
                if (ip.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
                {
                    this.ip = ip;
                    break;
                }
            }

            treeClients.IsExpanded = true;

            server = new Server();
            server.ip = this.ip;
            server.port = this.port;
            serverThread = new Thread(server.Run);
            serverThread.IsBackground = true;
            serverThread.Start();

            labelIP.Content = this.ip.ToString();
            labelPort.Content = this.port.ToString();

        }

        public void updateTime() {
            DateTime now = DateTime.Now;
            timeHour.Content = (now.Hour < 10 ? "0" : "") + now.Hour.ToString();
            timeMinute.Content = (now.Minute < 10 ? "0" : "") + now.Minute.ToString();
            timeSecond.Content = (now.Second < 10 ? "0" : "") + now.Second.ToString();
        }

        public void display() {
            //vypis klientu
            treeClients.Items.Clear();
            treeClientsLabel.Content = "Klienti (" + server.Clients.Count.ToString() + ")";
            List<Client> clients = server.Clients;
            foreach (Client client in clients) {
                Ellipse e = new Ellipse();
                e.Width = 8;
                e.Height = 8;
                BrushConverter bc = new BrushConverter();
                e.Stroke = (Brush)bc.ConvertFrom(client.Color);
                e.Fill = (Brush)bc.ConvertFrom(client.Color);
                Label l = new Label();
                l.Content = client.Name;
                l.Foreground = Brushes.White;
                Grid item = new Grid();
                ColumnDefinition cd = new ColumnDefinition();
                cd.Width = new GridLength(10);
                item.ColumnDefinitions.Add(cd);
                ColumnDefinition cd2 = new ColumnDefinition();
                item.ColumnDefinitions.Add(cd2);
                Grid.SetColumn(e, 0);
                Grid.SetColumn(l, 1);
                item.Children.Add(e);
                item.Children.Add(l);
                treeClients.Items.Add(item);
            }

            if (this.map != null)
            {
                if (this.map.isDisplay())
                {
                    overlay.Visibility = Visibility.Collapsed;
                    mapWindow.Children.Clear();
                    Canvas canvas = new Canvas();
                    if (showMousePosition) {
                        double x = Math.Round((MousePosition.X - this.map.posunX) / this.map.Meritko);
                        double y = Math.Round((MousePosition.Y - this.map.posunY) / this.map.Meritko);
                        labelStatus.Content = "X: " + x.ToString() + " | Y: " + y.ToString();
                    }
                    this.map.display(this.mapWindow, canvas, server.Clients);
                    mapWindow.Children.Add(canvas);                    
                }
                else {
                    labelOverlay.Content = "Nic k zobrazení...";
                    overlay.Visibility = Visibility.Visible;
                }
            }
        }

        private void MenuItemTemplateDark_Click(object sender, RoutedEventArgs e)
        {
            MenuItemTemplateLight.IsChecked = false;
            BrushConverter bc = new BrushConverter();
            menu.Background = (Brush)bc.ConvertFrom("#FF2C2C31");
            MenuItemFile.Foreground = Brushes.White;
            MenuItemTemplate.Foreground = Brushes.White;
            MenuItemInfo.Foreground = Brushes.White;
            bottomPanel.Background = (Brush)bc.ConvertFrom("#FF2C2C31");
            labelStatus.Foreground = Brushes.White;
            labelAuthor.Foreground = Brushes.White;
            overlay.Background = (Brush)bc.ConvertFrom("#FF73757F");
            labelOverlay.Foreground = Brushes.White;
            leftGrid.Background = (Brush)bc.ConvertFrom("#FF4A4A4F");
            timeHour.Foreground = Brushes.White;
            timeMinute.Foreground = Brushes.White;
            timeSecond.Foreground = Brushes.White;
            label2.Foreground = Brushes.White;
            label4.Foreground = Brushes.White;
            treeView.Foreground = Brushes.White;
            foreach (TreeViewItem t in treeView.Items) {
                t.Foreground = Brushes.White;
                foreach (TreeViewItem l in t.Items) {
                    l.Foreground = Brushes.White;
                }
            }
        }

        private void MenuItemTemplateLight_Click(object sender, RoutedEventArgs e)
        {
            MenuItemTemplateDark.IsChecked = false;
            BrushConverter bc = new BrushConverter();
            menu.Background = (Brush)bc.ConvertFrom("#00D3D3CE");
            MenuItemFile.Foreground = Brushes.Black;
            MenuItemTemplate.Foreground = Brushes.Black;
            MenuItemInfo.Foreground = Brushes.Black;
            bottomPanel.Background = (Brush)bc.ConvertFrom("#00D3D3CE");
            labelStatus.Foreground = Brushes.Black;
            labelAuthor.Foreground = Brushes.Black;
            overlay.Background = (Brush)bc.ConvertFrom("#008C8A80");
            labelOverlay.Foreground = Brushes.Black;
            leftGrid.Background = (Brush)bc.ConvertFrom("#FFD1D1D1");
            timeHour.Foreground = Brushes.Black;
            timeMinute.Foreground = Brushes.Black;
            timeSecond.Foreground = Brushes.Black;
            label2.Foreground = Brushes.Black;
            label4.Foreground = Brushes.Black;
            treeView.Foreground = Brushes.Black;
            foreach (TreeViewItem t in treeView.Items)
            {
                t.Foreground = Brushes.Black;
                foreach (TreeViewItem l in t.Items)
                {
                    l.Foreground = Brushes.Black;
                }
            }
        }
        private void MenuItemSetting_Click(object sender, RoutedEventArgs e)
        {
            SettingWindow window = new SettingWindow();
            window.Owner = this;
            window.setInputIP(this.ip);
            window.Show();
        }

        private void MenuItemSignal_Click(object sender, RoutedEventArgs e)
        {
            SignalWindow signalWindow = new SignalWindow();
            signalWindow.Owner = this;
            signalWindow.setClientList(this.getClients());
            signalWindow.Show();
        }

        private void MenuItemInfo_Click(object sender, RoutedEventArgs e)
        {
            InfoWindow infoWindow = new InfoWindow();
            infoWindow.Show();
        }

        private void MenuItemFileClose_Click(object sender, RoutedEventArgs e) {
            this.Close();
        }

        public IPAddress getIP() {
            return this.ip;
        }

        public void setIP(IPAddress ip) {
            this.ip = ip;
            this.labelIP.Content = this.ip.ToString();
        }

        public void setPort(Int32 port) {
            this.port = port;
            this.labelPort.Content = port.ToString();
        }

        public void restartServer() {
            //serverThread.Abort();
            server = new Server();
            server.ip = this.ip;
            server.port = this.port;
            serverThread = new Thread(server.Run);
            serverThread.IsBackground = true;
            serverThread.Start();
        }

        public List<Client> getClients() {
           // if(this.server.Clients.Count > 0)
                return this.server.Clients;
           //return new List<Client>();
        }

        private void MenuItemFileOpen_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            
            dlg.DefaultExt = ".map";
            dlg.Filter = "Map Files (*.map)|*.map";

            Nullable<bool> result = dlg.ShowDialog();

            if (result == true)
            {
                labelOverlay.Content = "Načítání mapy...";
                overlay.Visibility = Visibility.Visible;

                string filename = dlg.FileName;
                if (this.map == null)
                    this.map = new Map(filename);
                else
                {
                    this.map.loadMap(filename);
                }
                mapName.Content = this.map.Name;
            }
        }

        public void OnWindowClosing(object sender, CancelEventArgs e)
        {
            server.Stop();
            Thread.Sleep(100);
            serverThread.Abort();
        }

        public static void showAlert(string title, string text) {
            MessageBox.Show(text,title,MessageBoxButton.OK,MessageBoxImage.Error);
        }

        private void mapCanvas_MouseEnter(object sender, MouseEventArgs e)
        {
            showMousePosition = true;
        }

        private void mapCanvas_MouseLeave(object sender, MouseEventArgs e)
        {
            showMousePosition = false;
            labelStatus.Content = "Připraven";
        }

        private void mapWindow_MouseMove(object sender, MouseEventArgs e)
        {
            MousePosition = e.GetPosition(mapWindow);
        }
    }
}
