﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

namespace WifiLocator
{
    public class Map
    {

        private bool allowDisplay = false;

        public Map(string file) {
            this.loadMap(file);
        }

        public void loadMap(string file) {
            try
            {   // Open the text file using a stream reader.
                using (StreamReader sr = new StreamReader(file))
                {
                    this.allowDisplay = false;
                    this.Materials = new List<Material>();
                    this.Walls = new List<Wall>();
                    this.Doors = new List<Door>();
                    this.Labels = new List<MLabel>();
                    this.SignalMaps = new List<SignalMap>();
                    String content = sr.ReadToEnd();
                    try
                    {
                        dynamic data = JsonConvert.DeserializeObject(content);
                        this.Name = data.name;
                        this.Width = data.width;
                        this.Height = data.height;

                        //materialy
                        foreach(dynamic m in data.materials)
                        {
                            Console.WriteLine("Material: {0}, {1}, {2}",m.id,m.name,m.color);
                            int id = m.id;
                            string name = m.name;
                            string color = m.color;
                            Material material = new Material(id, name, color);
                            this.Materials.Add(material);
                        }

                        //steny
                        foreach (dynamic w in data.walls) {
                            Console.WriteLine("Wall\n\tmaterial\t{0}\n\tpoint1\t\t({1};{2})\n\tpoint2\t\t({3};{4})\n\tdepth\t\t{5}",w.material,w.x1,w.y1,w.x2,w.y2,w.depth);
                            int material = w.material;
                            int x1 = w.x1;
                            int x2 = w.x2;
                            int y1 = w.y1;
                            int y2 = w.y2;
                            int d = w.depth;
                            Wall wall = new Wall(this.getMaterialById(material),x1,y1,x2,y2,d);
                            this.Walls.Add(wall);
                        }

                        //dvere
                        foreach (dynamic d in data.doors) {
                            Console.WriteLine("Door\n\tPoint1\t\t({0};{1})\n\tPointC\t\t({2};{3})\n\tPoint2\t\t({4};{5})",d.x1,d.y1,d.xc,d.yc,d.x2,d.y2);
                            int x1 = d.x1;
                            int xc = d.xc;
                            int x2 = d.x2;
                            int y1 = d.y1;
                            int yc = d.yc;
                            int y2 = d.y2;
                            Door door = new Door(x1,y1, xc, yc, x2, y2);
                            this.Doors.Add(door);
                        }

                        //popisky
                        foreach (dynamic d in data.labels)
                        {
                            Console.WriteLine("Label\n\tposition: ({0};{1})\n\ttext: {2}\n\tcolor: {3}\n\tsize: {4}",d.x,d.y,d.text,d.color,d.size);
                            int x = d.x;
                            int y = d.y;
                            int size = d.size;
                            string color = d.color;
                            string text = d.text;
                            MLabel label = new MLabel(x,y,text,size,color);
                            this.Labels.Add(label);
                        }

                        //signaly
                        DateTime ctime = DateTime.Now;
                        foreach (dynamic d in data.signals) {
                            string mac = d.mac;
                            SignalMap sm = new SignalMap(mac, this.Width, this.Height);
                            foreach (dynamic sp in d.points) {
                                int x = sp.x;
                                int y = sp.y;
                                int signal = sp.signal;
                                sm.setSignal(x, y, signal);
                            }
                            if (!sm.calculateSignals())
                                MainWindow.showAlert("Chyba","Při výpočtu signálu mapy ("+ mac + "), bylo dosaženo max. počtu výpočetních cyklů!");
                            this.SignalMaps.Add(sm);
                        }

                        Console.WriteLine("Time: "+ (DateTime.Now.Millisecond - ctime.Millisecond).ToString()+"ms");


                        if (this.Walls.Count > 0)
                            this.allowDisplay = true;
                        Console.WriteLine("Loading done.");
                    }
                    catch (Exception e) {
                        MainWindow.showAlert("Chyba načítání mapy", e.Message);
                    }
                }
            }
            catch (Exception e)
            {
                MainWindow.showAlert("Soubor nelze otevřít", e.Message);
            }
        }

        public void display(Grid window, Canvas canvas, List<Client> clients) {
            if (allowDisplay) {
                canvas.Children.Clear();

                //vypocet meritka

                this.Meritko = 1;

                double windowWidth = window.ColumnDefinitions[0].ActualWidth;
                double windowHeight = window.RowDefinitions[0].ActualHeight;

                posunX = 20;
                posunY = 20;

                double m1 = Math.Floor(((windowWidth - (posunX * 2)) / this.Width) * 1000) / 1000;
                double m2 = Math.Floor(((windowHeight - (posunY * 2)) / this.Height) * 1000) / 1000;

                if (m1 < m2)
                {
                    this.Meritko = Math.Abs(m1);
                    posunY += (((m2 * this.Height) - (m1 * this.Height)) / 2);
                }
                else if (m1 > m2)
                {
                    this.Meritko = Math.Abs(m2);
                    posunX += (((m1 * this.Width) - (m2 * this.Width)) / 2);
                }
                else {
                    this.Meritko = Math.Abs(m1);
                }

                //zobrazeni zdi
                foreach (Wall wall in this.Walls) {
                    wall.display(canvas, this.Meritko, posunX, posunY);
                }

                //zobrazeni dveri
                foreach (Door door in this.Doors) {
                    door.display(canvas, this.Meritko, posunX, posunY);
                }

                //zobrazeni napisu
                foreach (MLabel label in this.Labels) {
                    label.display(canvas, this.Meritko, posunX, posunY);
                }

                //zobrazeni clientu
                foreach (Client client in clients) {
                    if (client.newData)
                    {
                        client.newData = false;
                        Console.WriteLine("Calculating new position {0}",client.Name);
                        client.calculatePosition(SignalMaps);
                    }
                    client.display(canvas, this.Meritko, posunX, posunY);
                }
            }
        }

        public Material getMaterialById(int id) {
            foreach (Material m in this.Materials) {
                if (m.ID == id)
                    return m;
            }
            return new Material(0, "Default", "#FF0000");
        }

        public string Name { get; set; }

        public int Width { get; set; }

        public int Height { get; set; }

        public double Meritko { get; set; }

        public double posunX { get; set; }

        public double posunY { get; set; }

        public bool isDisplay() {
            return this.allowDisplay;
        }

        List<Material> Materials { get; set; }

        List<Wall> Walls { get; set; }

        List<Door> Doors { get; set; }

        List<MLabel> Labels { get; set; }

        List<SignalMap> SignalMaps { get; set; }
    }
}
