﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;

namespace WifiLocator
{
    public class Server
    {

        private bool running;

        private Socket serverSocket;

        public IPAddress ip { get; set; }

        public Int32 port { get; set; }

        public List<Client> Clients { get; set; }

        private List<Thread> clientThreads = new List<Thread>();

        public List<string> clientNames { get; set; }

        public List<string> clientColors { get; set; }

        public Server() {
            Clients = new List<Client>();
            clientNames = new List<string>();
            clientColors = new List<string>();
            clientNames.Insert(0, "Client 1");
            clientNames.Insert(1, "Client 2");
            clientNames.Insert(2, "Client 3");
            clientColors.Insert(0, "#0000FF");
            clientColors.Insert(1, "#00FF00");
            clientColors.Insert(2, "#FF0000");
        }

        public void Run() {
            running = true;

            Clients = new List<Client>();
            try
            {
                serverSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                serverSocket.Bind(new IPEndPoint(this.ip, this.port));
                serverSocket.Listen(16);
                Console.WriteLine("Server ready");
                while (running)
                {
                    Console.WriteLine("Waiting on connection");
                    Socket clientSocket = serverSocket.Accept();
                    Console.WriteLine("New connection");
                    Thread t = new Thread(() => clientListener(clientSocket));
                    t.IsBackground = true;
                    int index = clientThreads.Count;
                    clientThreads.Insert(index, t);
                    clientThreads[index].Start();
                }
                Console.WriteLine("Server stopped");
            }
            catch (ThreadAbortException)
            {
                Console.WriteLine("Server stopped!");
            }
            catch (SocketException) {
                Console.WriteLine("Socket error!");
            }
            catch (Exception e)
            {
                MainWindow.showAlert("Chyba server", "Při běhu serveru nastala chyba: " + e.Message);
            }
        }

        public void Stop() {
            Console.WriteLine("STOP SERVER");
            running = false;
            serverSocket.Close();
            foreach (Thread t in clientThreads) {
                t.Abort();
            }
        }

        private void clientListener(Socket clientSocket) {
            Console.WriteLine("Client listener ready");
            int index = -1;
            try
            {
                bool run = true;
                while (run)
                {
                    byte[] idata = new byte[clientSocket.SendBufferSize];
                    int j = clientSocket.Receive(idata);
                    Console.WriteLine("New data");
                    byte[] buf = new byte[j];
                    for (int i = 0; i < j; i++)
                        buf[i] = idata[i];
                    string data = Encoding.Default.GetString(buf);
                    //Console.WriteLine(data);
                    if (!ServerExtension.IsConnected(clientSocket))
                        run = false;
                    else
                    {
                        string pattern = @"^(?<id>[0-9]+)\[(?<msg>[^\]]+)\]$";
                        Regex r = new Regex(pattern, RegexOptions.IgnoreCase);
                        Match m = r.Match(data);
                        if (m.Success) {
                            //parsovani ID zarizeni
                            string c = m.Groups["id"].Value;
                            int id = int.Parse(c);

                            string msg = m.Groups["msg"].Value;
                            
                            //nacteni objektu clienta
                            if (index < 0) {
                                int listindex = 0;
                                foreach (Client client in Clients) {
                                    if (client.Id == id) { 
                                        index = listindex;
                                        break;
                                    }
                                    listindex++;
                                }
                                if(index < 0) { 
                                    int i = id - 1;
                                    Client client = new WifiLocator.Client(id, clientNames[i],clientColors[i],0,0);
                                    index = Clients.Count;
                                    Clients.Insert(index, client);
                                }
                            }

                            if (index >= 0)
                            {
                                Clients[index].parseMessage(msg);
                            }
                        }
                    }
                    Console.WriteLine("Data end");
                }
            }
            catch (Exception e) {
                Console.WriteLine("Client listener error: " + e.Message);
            }
            Console.WriteLine("Client disconnected");
            if (index >= 0) {
                Clients.RemoveAt(index);
            }
        }
    }
}
