﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Threading;

namespace WifiLocator
{
    /// <summary>
    /// Interaction logic for SignalWindow.xaml
    /// </summary>
    public partial class SignalWindow : Window
    {

        ObservableCollection<SignalListItem> signalListItems = new ObservableCollection<SignalListItem>();
        bool loadingSignal = false;
        int secdelay = 0;
        int count = 0;
        DateTime lastLoad;
        DispatcherTimer timer;

        public SignalWindow()
        {
            InitializeComponent();

            signalList.ItemsSource = signalListItems;
        }

        public void setClientList(List<Client> clients) {
            foreach (Client client in clients)
            {
                clientComboBox.Items.Add(client.Name);
            }
        }

        private void btnSignalStart_Click(object sender, RoutedEventArgs e)
        {
            if (!loadingSignal)
            {
                this.loadingSignal = true;
                btnSignalStart.Content = "Stop";
                string client = clientComboBox.SelectedValue.ToString();
                Console.WriteLine("Selected client: " + client);
                count = 0;
                secdelay = 0;
                lastLoad = DateTime.Now;
                timer = new DispatcherTimer(new TimeSpan(0, 0, 1), DispatcherPriority.Normal, delegate
                {
                    this.loadSignals();
                }, this.Dispatcher);
            }
            else {
                this.loadingSignal = false;
                btnSignalStart.Content = "Start";
                timer.Stop();
            }
        }

        private void btnSignalReset_Click(object sender, RoutedEventArgs e)
        {
            secdelay = 0;
            count = 0;
            lastLoad = DateTime.Now;
            signalListItems.Clear();
            status.Content = "Počet načtení: "+count+" | Poslední načtení před " + secdelay.ToString() + "s";
        }

        private void loadSignals() {
            secdelay++;
            status.Content = "Počet načtení: "+count+" | Poslední načtení před "+secdelay.ToString()+"s";
            string client = clientComboBox.SelectedValue.ToString();
            List<Client> clients = ((MainWindow)this.Owner).getClients();
            foreach (Client c in clients) {
                if (c.Name == client) {
                    if (c.time > this.lastLoad)
                    {
                        Console.WriteLine("Display new data");
                        count++;
                        lastLoad = DateTime.Now;
                        secdelay = 0;
                        List<SignalListItem> signalListCache = new List<SignalListItem>();
                        foreach (Signal s in c.signals)
                        {
                            bool found = false;
                            foreach (SignalListItem si in this.signalListItems) {
                                if (si.Mac == s.MacAddress) {
                                    found = true;
                                    si.sum += s.Strenght;
                                    si.count++;
                                    si.Signal = si.sum / si.count;
                                    signalListCache.Add(si);
                                }
                            }
                            if(!found)
                                signalListCache.Add(new SignalListItem(s.SSID, s.MacAddress, s.Strenght));
                        }
                        signalListItems.Clear();
                        foreach (SignalListItem s in signalListCache) {
                            signalListItems.Add(s);
                        }
                    }
                }
            }
        }
    }
}
